package es.rickyepoderi.webcryptows;

import java.io.IOException;
import java.io.InputStream;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.Signature;
import java.security.UnrecoverableKeyException;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import javax.ws.rs.FormParam;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;
import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;
import sun.security.provider.X509Factory;

/**
 * <p>Simple WS class that say hello in several formats</p>
 * 
 * @author ricky
 */
@Path("/webcrypto")
public class WebCryptoWS {
    
    static public final String KEY_STORE_NAME = "/server.pkcs12";
    static public final String KEY_STORE_PASSWORD = "Kiosko_00";
    static public final String KEY_STORE_TYPE = "pkcs12";
    static public final String KEY_NAME = "testsign";
    static public final String KEY_PASSWORD = "Kiosko_00";
    static public final String SIGNATURE_METHOD = "SHA256withRSA";
    
    private Certificate certificate = null;
    private PrivateKey privateKey = null;
    
    public WebCryptoWS() {
        try (InputStream is = this.getClass().getResourceAsStream(KEY_STORE_NAME)) {
            // load the server private key to sign
            KeyStore keystore = KeyStore.getInstance(KEY_STORE_TYPE);
            keystore.load(is, KEY_STORE_PASSWORD.toCharArray());
            this.privateKey = (PrivateKey) keystore.getKey(KEY_NAME, KEY_PASSWORD.toCharArray());
            this.certificate = keystore.getCertificate(KEY_NAME);
        } catch (IOException | KeyStoreException | NoSuchAlgorithmException | CertificateException | UnrecoverableKeyException e) {
            e.printStackTrace();
        }
    }
    
    public String getCertificate() throws Exception {
        return new StringBuilder(X509Factory.BEGIN_CERT)
                .append(System.lineSeparator())
                .append(new BASE64Encoder().encode(certificate.getEncoded()))
                .append(X509Factory.END_CERT)
                .toString();
    }
    
    public String sign(String text) throws Exception {
        Signature sig = Signature.getInstance(SIGNATURE_METHOD);
        sig.initSign(this.privateKey);
        sig.update(text.getBytes());
        byte[] signatureBytes = sig.sign();
        String res = new BASE64Encoder().encode(signatureBytes);
        return res;
    }
    
    public boolean validate(String text, String signature) throws Exception {
        Signature sig = Signature.getInstance(SIGNATURE_METHOD);
        byte[] signatureBytes = new BASE64Decoder().decodeBuffer(signature);
        sig.initVerify(this.certificate.getPublicKey());
        sig.update(text.getBytes());
        return sig.verify(signatureBytes);
    }
    
    @GET
    @Path("/certificate")
    @Produces(MediaType.TEXT_PLAIN)
    public String getCertificateGet() throws Exception {
        return getCertificate();
    }

    @GET
    @Path("/sign")
    @Produces(MediaType.TEXT_PLAIN)
    public String signGet(@QueryParam("text") String text) throws Exception {
        return sign(text);
    }
    
    @POST
    @Path("/sign")
    @Produces(MediaType.TEXT_PLAIN)
    public String signPost(@FormParam("text") String text) throws Exception {
        return sign(text);
    }

    @GET
    @Path("/validate")
    @Produces(MediaType.TEXT_PLAIN)
    public boolean validateGet(
            @QueryParam("text") String text,
            @QueryParam("signature") String signature) throws Exception {
        return validate(text, signature);
    }
    
    @POST
    @Path("/validate")
    @Produces(MediaType.TEXT_PLAIN)
    public boolean validatePost(
            @FormParam("text") String text,
            @FormParam("signature") String signature) throws Exception {
        return validate(text, signature);
    }
}
